/*
 * * Copyright (C) 2009-2011 Ali <aliov@xfce.org>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __XFPM_ICONS_H
#define __XFPM_ICONS_H

G_BEGIN_DECLS

#define XFPM_AC_ADAPTER_ICON "ac-adapter"
#define XFPM_BATTERY_LEVEL_ICON "battery-level"
#define XFPM_COMPUTER_ICON "computer"
#define XFPM_DISPLAY_BRIGHTNESS_ICON "display-brightness-symbolic"
#define XFPM_MOUSE_ICON "input-mouse"
#define XFPM_KBD_ICON "input-keyboard"
#define XFPM_MEDIA_PLAYER_ICON "multimedia-player"
#define XFPM_PDA_ICON "pda"
#define XFPM_PHONE_ICON "phone"
#define XFPM_TABLET_ICON "tablet"
#define XFPM_UPS_ICON "uninterruptible-power-supply"
#define XFPM_MONITOR_ICON "video-display"
#define PANEL_DEFAULT_ICON "battery-full-charged"
#define PANEL_DEFAULT_ICON_SYMBOLIC "battery-full-charged-symbolic"
#define PRESENTATION_MODE_ICON "x-office-presentation-symbolic"

G_END_DECLS

#endif
