
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include <xfpm-enum-types.h>
#include "xfpm-enum-glib.h"

/* enumerations from "./xfpm-enum-glib.h" */
GType
xfpm_battery_charge_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
	static const GEnumValue values[] = {
	{ XFPM_BATTERY_CHARGE_UNKNOWN, "XFPM_BATTERY_CHARGE_UNKNOWN", "xfpm-battery-charge-unknown" },
	{ XFPM_BATTERY_CHARGE_CRITICAL, "XFPM_BATTERY_CHARGE_CRITICAL", "xfpm-battery-charge-critical" },
	{ XFPM_BATTERY_CHARGE_LOW, "XFPM_BATTERY_CHARGE_LOW", "xfpm-battery-charge-low" },
	{ XFPM_BATTERY_CHARGE_OK, "XFPM_BATTERY_CHARGE_OK", "xfpm-battery-charge-ok" },
	{ N_XFPM_BATTERY_CHARGES, "N_XFPM_BATTERY_CHARGES", "n-xfpm-battery-charges" },
	{ 0, NULL, NULL }
	};
	type = g_enum_register_static ("XfpmBatteryCharge", values);
  }
	return type;
}
GType
xfpm_shutdown_request_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
	static const GEnumValue values[] = {
	{ XFPM_DO_NOTHING, "XFPM_DO_NOTHING", "xfpm-do-nothing" },
	{ XFPM_DO_SUSPEND, "XFPM_DO_SUSPEND", "xfpm-do-suspend" },
	{ XFPM_DO_HIBERNATE, "XFPM_DO_HIBERNATE", "xfpm-do-hibernate" },
	{ XFPM_ASK, "XFPM_ASK", "xfpm-ask" },
	{ XFPM_DO_SHUTDOWN, "XFPM_DO_SHUTDOWN", "xfpm-do-shutdown" },
	{ XFPM_DO_HYBRID_SLEEP, "XFPM_DO_HYBRID_SLEEP", "xfpm-do-hybrid-sleep" },
	{ N_XFPM_SHUTDOWN_REQUESTS, "N_XFPM_SHUTDOWN_REQUESTS", "n-xfpm-shutdown-requests" },
	{ 0, NULL, NULL }
	};
	type = g_enum_register_static ("XfpmShutdownRequest", values);
  }
	return type;
}
GType
xfpm_lid_trigger_action_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
	static const GEnumValue values[] = {
	{ LID_TRIGGER_DPMS, "LID_TRIGGER_DPMS", "lid-trigger-dpms" },
	{ LID_TRIGGER_SUSPEND, "LID_TRIGGER_SUSPEND", "lid-trigger-suspend" },
	{ LID_TRIGGER_HIBERNATE, "LID_TRIGGER_HIBERNATE", "lid-trigger-hibernate" },
	{ LID_TRIGGER_LOCK_SCREEN, "LID_TRIGGER_LOCK_SCREEN", "lid-trigger-lock-screen" },
	{ LID_TRIGGER_NOTHING, "LID_TRIGGER_NOTHING", "lid-trigger-nothing" },
	{ LID_TRIGGER_HYBRID_SLEEP, "LID_TRIGGER_HYBRID_SLEEP", "lid-trigger-hybrid-sleep" },
	{ LID_TRIGGER_SHUTDOWN, "LID_TRIGGER_SHUTDOWN", "lid-trigger-shutdown" },
	{ N_LID_TRIGGERS, "N_LID_TRIGGERS", "n-lid-triggers" },
	{ 0, NULL, NULL }
	};
	type = g_enum_register_static ("XfpmLidTriggerAction", values);
  }
	return type;
}
GType
xfpm_button_key_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
	static const GEnumValue values[] = {
	{ BUTTON_UNKNOWN, "BUTTON_UNKNOWN", "button-unknown" },
	{ BUTTON_POWER_OFF, "BUTTON_POWER_OFF", "button-power-off" },
	{ BUTTON_HIBERNATE, "BUTTON_HIBERNATE", "button-hibernate" },
	{ BUTTON_SLEEP, "BUTTON_SLEEP", "button-sleep" },
	{ BUTTON_MON_BRIGHTNESS_UP, "BUTTON_MON_BRIGHTNESS_UP", "button-mon-brightness-up" },
	{ BUTTON_MON_BRIGHTNESS_DOWN, "BUTTON_MON_BRIGHTNESS_DOWN", "button-mon-brightness-down" },
	{ BUTTON_LID_CLOSED, "BUTTON_LID_CLOSED", "button-lid-closed" },
	{ BUTTON_BATTERY, "BUTTON_BATTERY", "button-battery" },
	{ BUTTON_KBD_BRIGHTNESS_UP, "BUTTON_KBD_BRIGHTNESS_UP", "button-kbd-brightness-up" },
	{ BUTTON_KBD_BRIGHTNESS_DOWN, "BUTTON_KBD_BRIGHTNESS_DOWN", "button-kbd-brightness-down" },
	{ BUTTON_KBD_BRIGHTNESS_CYCLE, "BUTTON_KBD_BRIGHTNESS_CYCLE", "button-kbd-brightness-cycle" },
	{ N_XFPM_BUTTON_KEYS, "N_XFPM_BUTTON_KEYS", "n-xfpm-button-keys" },
	{ 0, NULL, NULL }
	};
	type = g_enum_register_static ("XfpmButtonKey", values);
  }
	return type;
}
GType
xfpm_spindown_request_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
	static const GEnumValue values[] = {
	{ SPIN_DOWN_HDD_NEVER, "SPIN_DOWN_HDD_NEVER", "spin-down-hdd-never" },
	{ SPIN_DOWN_HDD_ON_BATTERY, "SPIN_DOWN_HDD_ON_BATTERY", "spin-down-hdd-on-battery" },
	{ SPIN_DOWN_HDD_PLUGGED_IN, "SPIN_DOWN_HDD_PLUGGED_IN", "spin-down-hdd-plugged-in" },
	{ SPIN_DOWN_HDD_ALWAYS, "SPIN_DOWN_HDD_ALWAYS", "spin-down-hdd-always" },
	{ N_SPIN_DOWN_HDDS, "N_SPIN_DOWN_HDDS", "n-spin-down-hdds" },
	{ 0, NULL, NULL }
	};
	type = g_enum_register_static ("XfpmSpindownRequest", values);
  }
	return type;
}
GType
xfpm_panel_label_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
	static const GEnumValue values[] = {
	{ PANEL_LABEL_NONE, "PANEL_LABEL_NONE", "panel-label-none" },
	{ PANEL_LABEL_PERCENTAGE, "PANEL_LABEL_PERCENTAGE", "panel-label-percentage" },
	{ PANEL_LABEL_TIME, "PANEL_LABEL_TIME", "panel-label-time" },
	{ PANEL_LABEL_PERCENTAGE_AND_TIME, "PANEL_LABEL_PERCENTAGE_AND_TIME", "panel-label-percentage-and-time" },
	{ N_PANEL_LABELS, "N_PANEL_LABELS", "n-panel-labels" },
	{ 0, NULL, NULL }
	};
	type = g_enum_register_static ("XfpmPanelLabel", values);
  }
	return type;
}

/* Generated data ends here */

